import requests
from bs4 import BeautifulSoup
from urllib.parse import urljoin, urlparse

def is_valid(url):
    """
    بررسی معتبر بودن URL.
    """
    parsed = urlparse(url)
    return bool(parsed.netloc) and bool(parsed.scheme)

def get_all_website_links(url, domain_name):
    """
    گرفتن تمام لینک‌های موجود در یک صفحه وب.
    """
    urls = set()
    try:
        response = requests.get(url, timeout=10)
        soup = BeautifulSoup(response.text, "html.parser")
        
        for a_tag in soup.find_all("a"):
            href = a_tag.attrs.get("href")
            if href == "" or href is None:
                continue
            href = urljoin(url, href)
            parsed_href = urlparse(href)
            href = parsed_href.scheme + "://" + parsed_href.netloc + parsed_href.path
            if not is_valid(href):
                continue
            if href in urls:
                continue
            if domain_name not in href:  # لینک خارجی
                print(f"لینک خارجی: {href}")
            else:  # لینک داخلی
                urls.add(href)
    except Exception as e:
        print(f"خطا در دریافت صفحه {url}: {e}")
    return urls

def crawl(url, max_pages=20000):
    """
    کراول کردن سایت و یافتن تمام لینک‌ها.
    """
    domain_name = urlparse(url).netloc
    visited = set()  # لینک‌هایی که قبلاً دیده شده‌اند
    to_visit = set([url])  # لینک‌هایی که باید باز شوند
    
    while to_visit and len(visited) < max_pages:
        current_url = to_visit.pop()
        visited.add(current_url)
        print(f"در حال بررسی: {current_url}")
        links = get_all_website_links(current_url, domain_name)
        for link in links:
            if link not in visited and link not in to_visit:
                to_visit.add(link)

if __name__ == "__main__":
    website_url = "https://avaydel.ir/"  # آدرس سایت را اینجا قرار دهید
    crawl(website_url)